CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C                  projet Estime, INRIA-Rocquencourt                  C
C                                                                     C
C  Copyright 2004-2011 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: memory.F,v 1.4 2011-03-09 12:50:15 fclement Exp $

C     
C     Routines to handle the low-level memory.
C     
      
#include "pio_constants.h"
      
      subroutine check_memory_invariant(sbr, io)
C     Check the invariant on memory pointers.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io
      
C     Local variables.
      logical out_of_bounds, invalid_data_access, overflow
      integer fp, sp
      
      fp = fetch_ptr(io)
      sp = store_ptr(io)
      
      out_of_bounds = fp .lt. MIN_MEMORY_LOCATION
      invalid_data_access = fp .gt. sp
      overflow = sp .gt. MAX_MEMORY_LOCATION
      
      if (out_of_bounds) then
         call error_out_of_bounds(sbr, io, 'fetch memory pointer')
      endif
      
      if (invalid_data_access) then
         call error_invalid_data_access
     +        (sbr, io, 'memory', 'fetch memory pointer', sp, fp)
      endif
      
      if (overflow) then
         call error_overflow(sbr, io, 'memory')
      endif
      
      return
      end
      
      
      integer function get_store_ptr(io)
C     Get the store pointer.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input argument.
      integer io
      
      get_store_ptr = store_ptr(io)
      
      return
      end
      
      
      subroutine set_store_ptr(io, loc)
C     Set the store pointer to loc.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      integer io, loc
      
      store_ptr(io) = loc
      
      return
      end
      
      
      integer function get_fetch_ptr(io)
C     Get the fetch pointer.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input argument.
      integer io
      
      get_fetch_ptr = fetch_ptr(io)
      
      return
      end
      
      
      subroutine set_fetch_ptr(io, loc)
C     Set the fetch pointer to loc.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      integer io, loc
      
      fetch_ptr(io) = loc
      
      return
      end
      
      
      logical function get_memory_clean(io)
C     Get the memory_clean flag value.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input argument.
      integer io
      
      get_memory_clean = memory_clean(io)
      
      return
      end
      
      
      subroutine set_memory_clean(io, b)
C     Set the memory_clean flag to b.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      logical b
      integer io
      
      memory_clean(io) = b
      
      return
      end
      
      
      subroutine check_memory_clean(sbr, io, good)
C     Check that the memory_clean value is good.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      logical good
      character sbr*(*)
      integer io
      
C     Local variable.
      logical bad
      
C     External function.
      logical get_memory_clean
      
      bad = get_memory_clean(io)
      if (bad .neqv. good) then
         call error_wrong_memory_clean(sbr, io, good, bad)
      endif
      
      return
      end
      
      
      subroutine advance_store_ptr(io, len)
C     Advance the store pointer of len locations.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      integer io, len
      
C     External function.
      logical pio_get_debug_mode
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *)
     +        'Before advance, store_ptr(', io, ') = ', store_ptr(io)
      endif
      
      store_ptr(io) = store_ptr(io) + len
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *)
     +        'After advance, store_ptr(', io, ') = ', store_ptr(io)
      endif
      
      return
      end
      
      
      subroutine advance_fetch_ptr(io, len)
C     Advance the fetch pointer of len locations.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      integer io, len
      
C     External function.
      logical pio_get_debug_mode
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *)
     +        'Before advance, fetch_ptr(', io, ') = ', fetch_ptr(io)
      endif
      
      fetch_ptr(io) = fetch_ptr(io) + len
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *)
     +        'After advance, fetch_ptr(', io, ') = ', fetch_ptr(io)
      endif
      
      return
      end
      
      
      subroutine init_memory(sbr, io)
C     Initialize the memory.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io
      
      store_ptr(io) = MIN_MEMORY_LOCATION
      fetch_ptr(io) = MIN_MEMORY_LOCATION
      
      call check_memory_invariant(sbr, io)
      
      call set_memory_clean(io, .False.)
      
      return
      end
      
      
      subroutine verify_lower_bound(sbr, io, loc)
C     Cannot access before the memory lower bound.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc
      
      if (loc .lt. MIN_MEMORY_LOCATION) then
         call error_memory_access(sbr, io, loc)
      endif
      
      return
      end
      
      
      subroutine verify_rand_fetch(sbr, io, loc, end_loc)
C     Cannot fetch out of the memory bounds.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, end_loc
      
      call verify_lower_bound(sbr, io, loc)
      
      if (loc .gt. end_loc) then
         call error_rand_fetch_range_memory_access
     +        (sbr, io, loc, end_loc)
      endif
      
      if (end_loc .gt. MAX_MEMORY_LOCATION) then
         call error_rand_fetch_memory_access
     +        (sbr, io, loc, end_loc)
      endif
      
      return
      end
      
      
      subroutine verify_rand_store(sbr, io, loc, end_loc)
C     Cannot store out of the memory bounds.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc, end_loc
      
      call verify_lower_bound(sbr, io, loc)
      
      if (loc .gt. end_loc) then
         call error_rand_store_range_memory_access
     +        (sbr, io, loc, end_loc)
      endif
      
      if (end_loc .gt. MAX_MEMORY_LOCATION) then
         call error_rand_store_memory_access
     +        (sbr, io, loc, end_loc)
      endif
      
      return
      end
      
      
      subroutine verify_seq_fetch_precondition(sbr, io, len)
C     Cannot fetch where nothing has ever been written.
      
      implicit none
      
      include 'memory_definition.fh'
      
C     Input arguments.
      character sbr*(*)
      integer io, len
      
C     fetch_ptr(io) + len - 1 <= store_ptr(io) - 1,
      if (fetch_ptr(io) + len .gt. store_ptr(io)) then
         call error_seq_fetch_memory_access
     +        (sbr, io, fetch_ptr(io), fetch_ptr(io) + len - 1)
      endif
      
      return
      end
